'' +--------------------------------------------------------------------------+
'' | Cluso's RamBlade & TriBladeProp:  PropBootSD (boot from SD card)   v1.10 |
'' +--------------------------------------------------------------------------+
'' |  Author:            "Cluso99" (Ray Rodrick)                              |
'' |  Copyright (c) 2010 "Cluso99" (Ray Rodrick)                              |
'' |  License            MIT License - See end of file for terms of use       |
'' +--------------------------------------------------------------------------+
                             
'' Thanks to Jeff Ledger for PropDOS, Mike Green & Tomas Rokicki for modified Fsrw code (from FemtoBASIC)

'' This is a minimal eeprom boot code to automatically boot a Propeller Binary
''   from the file "AUTOEXEC.BIN" on the SD card.
''   If "AUTOEXEC.BAT" is not found then "BOOTPROP.BIN" will be loaded.
''   If "BOOTPROP.BIN" is not found then the prop will shut down.
''
'' It is assumed there is no I/O so no messages are output.
''
'' RR20100108   PropBootSD_V1_00  minimal boot version (fixed DO tristate when booting)
'' RR20120120   PropBootSD_V1_10  mod for RamBlade3
'' RR20120602                     RamBlade1

CON

'Uncomment the following for the TriBlade
{
  _clkmode  = xtal1 + pll16x
' _xinfreq  = 5_000_000
  _xinfreq  = 6_000_000
  spiDO     = 9                 'microSD
  spiClk    = 28
  spiDI     = 8
  spiCS     = 14
  SIpin     = 31                'serial
  SOpin     = 30
  baud      = 115200
}

'Uncomment the following for the RamBlade

  _clkmode  = xtal1 + pll16x
  _xinfreq  = 6_500_000
  spiDO     = 24                'microSD
  spiClk    = 26
  spiDI     = 25
  spiCS     = 19
  SIpin     = 23                'serial
  SOpin     = 22
  baud      = 115200

{
'Uncomment the following for the RamBlade3

  _clkmode  = xtal1 + pll16x
  _xinfreq  = 6_500_000
  spiDO     = 20                'microSD
  spiClk    = 21
  spiDI     = 22
  spiCS     = 23
  SIpin     = 31                'serial
  SOpin     = 30
  baud      = 115200
}
      
OBJ
     fsrw  : "fsrwFemto_rr005"          'modified SD routines
'    uart  : "FullDuplexSerial_rr004"
      
VAR
     long   ioControl[2]
     byte   buff[512]
     
PUB mainProgram | r, t 

' uart.start(SIpin,SOpin,0,baud)
' waitcnt(clkfreq * 2 + cnt)
' crlf  
' uart.str(string("RamBlade1-3/TriBlade Boot Loader V1.10"))
' crlf
  
  r := \DoLoader                                              'find file(s) on microSD and boot

'here only if it fails   
' crlf
' uart.str(string("** Boot failed **"))
' crlf
  waitcnt(clkfreq * 5 + cnt)                            
  repeat t from 0 to 7                                   'Stop all COGs except this one
    if (t <> cogid)
      cogstop(t)
  cogstop(cogid)                                        'then stop this cog. All done!


PRI DoLoader | r                                     

  r := fsrw.start(@ioControl)                           'start I2C/SPI driver

  if \fsrw.mount(spiDO,spiClk,spiDI,spiCS) < 0          'check for SD card, abort if not found.
     abort 

  r := OpenFile(string("AUTOEXEC.BAT"))                 'open and read "autoexec.bat"
  if r == 0
    r := OpenFile(@buff)                                'open binary boot file
  if r <> 0                                             'not found (autoexec.bat or invalid file parameter)
    r := OpenFile(string("BOOTPROP.BIN"))               'open "bootprop.bin"
  if r == 0                                             'boot the binary file
'   uart.str(string("Booting file "))
'   waitcnt(clkfreq / 2 + cnt)                            
    fsrw.bootSDCard
  if r <> 0
    abort

PRI OpenFile (st) | r, fbase, fsize

' waitcnt(clkfreq * 2 + cnt)                            
' uart.str(string("Opening file "))
' uart.str(st)
' uart.str(string("   "))

  r := fsrw.popen(st,"r")                               'Open file for read (blank image)
  if r == 0
    fbase := fsrw.datablock                             'get the sectorno of the file
    r := fsrw.readSDCard(fbase, @buff, 128)             'get 128 bytes filename & params
    if r == 0
      buff[12+1+32] := 0                                'terminate filename/parameters string
'     UART.str(@buff)
      buff[12] := 0                                     'terminate filename string 8.3
  crlf
  result := r

PRI crlf
' uart.str(string(13,10))



dat                                                   
{{
+------------------------------------------------------------------------------------------------------------------------------+
|                                                   TERMS OF USE: MIT License                                                  |                                                            
+------------------------------------------------------------------------------------------------------------------------------+
|Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation    | 
|files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy,    |
|modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software|
|is furnished to do so, subject to the following conditions:                                                                   |
|                                                                                                                              |
|The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.|
|                                                                                                                              |
|THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE          |
|WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR         |
|COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,   |
|ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                         |
+------------------------------------------------------------------------------------------------------------------------------+
}} 